@@ -63,7 +63,7 @@ class NovaClientPlugin(microversion_mixin.MicroversionMixin,
 
     NOVA_API_VERSION = '2.1'
 
-    max_microversion = None
+    max_microversion = cfg.CONF.max_nova_api_microversion
 
     service_types = [COMPUTE] = ['compute']
 
@@ -169,6 +169,11 @@ class NovaClientPlugin(microversion_mixin.MicroversionMixin,
                 raise
         return server
 
+    def fetch_server_attr(self, server_id, attr):
+        server = self.fetch_server(server_id)
+        fetched_attr = getattr(server, attr, None)
+        return fetched_attr
+
     def refresh_server(self, server):
         """Refresh server's attributes.
 
@@ -308,7 +313,7 @@ class NovaClientPlugin(microversion_mixin.MicroversionMixin,
 
     def build_userdata(self, metadata, userdata=None, instance_user=None,
                        user_data_format='HEAT_CFNTOOLS'):
-        """Build multipart data blob for CloudInit.
+        """Build multipart data blob for CloudInit and Ignition.
 
         Data blob includes user-supplied Metadata, user data, and the required
         Heat in-instance configuration.
@@ -330,6 +335,10 @@ class NovaClientPlugin(microversion_mixin.MicroversionMixin,
         is_cfntools = user_data_format == 'HEAT_CFNTOOLS'
         is_software_config = user_data_format == 'SOFTWARE_CONFIG'
 
+        if (is_software_config and
+                NovaClientPlugin.is_ignition_format(userdata)):
+            return NovaClientPlugin.build_ignition_data(metadata, userdata)
+
         def make_subpart(content, filename, subtype=None):
             if subtype is None:
                 subtype = os.path.splitext(filename)[0]
@@ -428,6 +437,51 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
 
         return mime_blob.as_string()
 
+    @staticmethod
+    def is_ignition_format(userdata):
+        try:
+            payload = jsonutils.loads(userdata)
+            ig = payload.get("ignition")
+            return True if ig and ig.get("version") else False
+        except Exception:
+            return False
+
+    @staticmethod
+    def build_ignition_data(metadata, userdata):
+        if not metadata:
+            return userdata
+
+        payload = jsonutils.loads(userdata)
+        encoded_metadata = urlparse.quote(jsonutils.dumps(metadata))
+        path_list = ["/var/lib/heat-cfntools/cfn-init-data",
+                     "/var/lib/cloud/data/cfn-init-data"]
+        ignition_format_metadata = {
+            "filesystem": "root",
+            "group": {"name": "root"},
+            "path": "",
+            "user": {"name": "root"},
+            "contents": {
+                "source": "data:," + encoded_metadata,
+                "verification": {}},
+            "mode": 0o640
+        }
+
+        for path in path_list:
+            storage = payload.setdefault('storage', {})
+            try:
+                files = storage.setdefault('files', [])
+            except AttributeError:
+                raise ValueError('Ignition "storage" section must be a map')
+            else:
+                try:
+                    data = ignition_format_metadata.copy()
+                    data["path"] = path
+                    files.append(data)
+                except AttributeError:
+                    raise ValueError('Ignition "files" section must be a list')
+
+        return jsonutils.dumps(payload)
+
     def check_delete_server_complete(self, server_id):
         """Wait for server to disappear from Nova."""
         try:
@@ -513,6 +567,10 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
             return True
         if status == 'VERIFY_RESIZE':
             return False
+        task_state_in_nova = getattr(server, 'OS-EXT-STS:task_state', None)
+        # Wait till move out from any resize steps (including resize_finish).
+        if task_state_in_nova is not None and 'resize' in task_state_in_nova:
+            return False
         else:
             msg = _("Confirm resize for server %s failed") % server_id
             raise exception.ResourceUnknownStatus(
